

  CREATE OR REPLACE PROCEDURE HR_RPT_SNAPSHOT

-----------------------------------------------------------------------------
-- File:
--    hr_rpt_snapshot.sql
--
-- Description:
--    This proc is used to populate the FCM_HOURLY_SNAPSHOT table, which is
--    used in a number of reports.
--
--    Below is a description of each of the inline views and the variables they calculate that
--    either correspond directly to a metric used in the report or are used to construct that
--    metric. Each inline view calculates its metrics at the strategy/queue level.
--
--    INLINE VIEW: unschTypeii
--       METRICS:
--          unscheduled_type_ii - The Unscheduled metric used in the report is broken
--             down into three separate categories.  Refer to the reporting documentation for
--             details.
--       ALGORITHM:
--          Within each queue, determine the latest time (within the user-specified date range)
--          a unique alert was statused. For each unique alert returned from the prior set, find
--          the action that took place just prior to the latest status. With this information,
--          we can check whether or not the alert satisfies the type ii unscheduled definition.
--
--    INLINE VIEW: SchedAndUnschedTypes
--       METRICS:
--          Scheduled_type_ii_plusUnsch_ii - This metric counts all unique alerts that would qualify
--             as both Scheduled type ii and Unscheduled type ii.  It will be used to calculate
--             the Scheduled metric.
--          serviceableReviewed - Alerts that have been statused but at the end of the time period
--             are again serviceable contribute to the ScheduledReviewed metric.
--          Unscheduled_types_i_and_iii - This metric counts all unique alerts that satisfy
--             Unscheduled type ii and Unscheduled type iii for the given queue within the time period.
--          Actioned - This is the count of unique alerts that were actioned in the given queue
--             within the given time period
--       ALGORITHM:
--          We essentially want to surface every alert that was statused and know the queue in which it
--          was statused. However, in order to know how to categorize them properly, we have to
--          know the status of the alert at the end of the time interval. It's quite possible that
--          at the end of the time interval, an alert will reside in a queue that is different from
--          the one in which it was statused. It's also possible for the same unique alert to be
--          statused in multiple different queues within the time interval. The SQL for this
--          inline view joins the two dynamically created tables (each containing their own dynamically
--          created tables): (1) dynamic table aa stores information about the last action record
--          in the time interval and (2) dynamic table latest stores the unique alerts within each queue
--          that was statused within the time interval.
--
--    INLINE VIEW: serviceableAlerts
--       METRICS:
--          serviceableAlertsCnt - The number of serviceable alerts at the end of the time period, broken
--             out by queue.
--          newAlertsCnt - The number of alerts in each queue at the end of the time interval that were
--             were created within the time interval. It's important to note that when the alert was
--             created, it didn't necessarily reside in the queue in which it's being reported.
--       ALGORITHM:
--          Extract the last action for each unique alert before the end of the time interval. For each
--          alert, determine which are serviceable and which are new from these records. Each alert that is
--          categorized as new/scheduled will contribute to the newAlertsCnt/serviceableAlertsCnt metric for
--          the queue that the alert was in at the end of the time interval.
--
--    INLINE VIEW: unresolvedActions
--       METRICS:
--          connectCnt - The number of times analysts statused alerts in a given queue within the time
--             interval
--          totalTimeHr - The total time taken by analysts statusing alerts within the time interval,
--             broken out by queue.
--          rescheduleCnt - The number of times analysts statused alerts as Reschedule in a given queue
--             within the time interval
--          skipCnt - The number of times analysts statused alerts as Skip in a given queue
--             within the time interval
--          pendCnt - The number of times analysts statused alerts as Pend in a given queue
--             within the time interval
--          pendBlockCnt - The number of times analysts statused alerts as Pend with block in a given queue
--             within the time interval
--          reRouteCnt - The number of times analysts Rerouted alerts *into* a given queue
--             within the time interval
--       ALGORITHM:
--          Tally all the "unresolved" alert statuses and the time taken to perform the statuses within
--          the time interval, breaking the values out by the queue in which they occurred.
--
--    INLINE VIEW: resolvedMetrics
--       METRICS:
--          verifiedActivity - Number of unique alerts within a given queue that had a *final status* of
--             Verified Activity by the end of the time interval.
--          confirmedFraud - Number of unique alerts within a given queue that had a *final status* of
--             Confirmed Fraud by the end of the time interval.
--          prevented - The amount of remaining credit in a given queue on accounts that had a
--             *final status* of Confirmed Fraud by the end of the time interval.
--       ALGORITHM:
--          Extract the last status that took place on each unique alert within the time interval.
--          Join this set back to the appropriate tables to extract the needed information,
--          restricting the subset to only those statused as Verified Activity or Confirmed Fraud.
--
-- Called By:
--    Cron job that is scheduled to run hourly.
--
-- Calls:
--
--
-- Notes:
--    The proc is built so that it can backfill any hours that are missed
--    due to downtime. A maximum of 150 days can be backfilled.
--
-- Usage:
--
--
-- History:
--     2012/07/23    leushe				 Ported from DB2
--     2014/06/19    leushe				 Fixed DATE issue, so DATE is a DATE not a timestamp
--
------------------------------------------------------------------------

IS

   l_last_date DATE;
   l_last_hr INTEGER;
   l_current_date DATE;
   l_current_hr INTEGER;

   l_snap_date DATE;
   l_snap_hr INTEGER;
   v_sqlstate char(5) DEFAULT '00000';
   l_row_count INTEGER;


  --DECLARE cursor1 CURSOR WITH RETURN FOR
  CURSOR cursor1  IS
    select snapDate, snapHour from
    (
		 select to_date(SYS_EXTRACT_UTC(CURRENT_TIMESTAMP) - 151 ) + level as snapDate
		        from DUAL     connect by level<=151		     
    )  snapDateQuery
    left outer join
    (
         select 0 as snapHour from dual
         union
         select 0 + level as snapHour
            from DUAL     connect by level<=23   
    )  snapHourQuery
    on 1=1
    where
        (
        snapDate = trunc(l_current_date) and snapDate <> trunc(l_last_date) and snapHour < l_current_hr or
        snapDate = trunc(l_last_date) and snapDate <> trunc(l_current_date) and snapHour > l_last_hr or
        snapDate > trunc(l_last_date) and snapDate < trunc(l_current_date) or
        snapDate = trunc(l_last_date) and snapDate = trunc(l_current_date) and snapHour > l_last_hr and snapHour < l_current_hr
        )
    ;

BEGIN

   -- step 1. set local variables
    l_current_hr := EXTRACT(HOUR FROM SYS_EXTRACT_UTC(CURRENT_TIMESTAMP));
    l_current_date := TO_DATE(TO_CHAR (SYS_EXTRACT_UTC(CURRENT_TIMESTAMP), 'YYYY-MON-DD'),'YYYY-MON-DD');





   --
   -- figure out the last time this thing ran
   --    this will establish the baseline for every subselect in the big insert statement
   --
   select count(*) into l_row_count from FCM_HOURLY_SNAPSHOT;
   IF ( l_row_count = 0 ) THEN
      l_last_date := TO_DATE(TO_CHAR (SYS_EXTRACT_UTC(CURRENT_TIMESTAMP)-150, 'YYYY-MON-DD'),'YYYY-MON-DD');
      l_last_hr := EXTRACT(HOUR FROM SYS_EXTRACT_UTC(CURRENT_TIMESTAMP));
   ELSE
     select max(coalesce(snap_date, l_current_date)) into l_last_date from FCM_HOURLY_SNAPSHOT;
     select max(coalesce(snap_hr, l_current_hr))
        into l_last_hr from FCM_HOURLY_SNAPSHOT where snap_date
        = ( select max(coalesce(snap_date, l_current_date)) from FCM_HOURLY_SNAPSHOT );
   END IF;

     l_last_date := trunc(l_last_date);

      OPEN cursor1;

--
--
        v_sqlstate := '00000';
        WHILE ( v_sqlstate = '00000' ) LOOP
            
            FETCH  cursor1 into
                l_snap_date, l_snap_hr ;

            IF ( cursor1%FOUND  ) THEN
                v_sqlstate := '00000';
                                --insert into fcm_hourly_snapshotk (snap_date,snap_hr) values ( l_snap_date,l_snap_hr);
                                --commit;

                INSERT INTO fcm_hourly_snapshot
   ( snap_date, snap_hr, snap_end_of_day_ind, multi_org_id, multi_org_name,
     strategy_name, queue_name, alert_type, alert_type_name, total_alert_count,
     scheduled_alert_count, caryover_alert_count,
     new_alert_count, unschd_alert_count, actioned_alert_count,
     schd_actioned_alert_count,
     penetration_rate, unschd_actioned_alert_count, resolved_alert_count,
     resolved_alert_rate, skip_count, reroute_count, reschd_count,
     pend_count, pend_block_count, verified_ok_count, confirmed_fraud_count,
     fp_ratio, saved_amount, total_hours_worked,
     serviceable, strategy_id, queue_id, analyst_action_count)
                WITH
                unschTypeii as (
                    select s.business_unit_id, faa.multi_org_id, s.strategy_id, q.queue_id, faa.alert_type,
                        sum(
                            case
                                when faa.next_service_timestamp > statusedAlerts.latestStatusTimestamp and faa.service_method_code = 'INB' and s.strategy_type_code = 'OUT'
                                    then 1
                                else 0
                            end
                        ) as unscheduled_type_ii
                    from fcm_alert_action faa inner join
                        (
                            select aa.alert_id, aa.create_timestamp, latestStatus.queue_id, latestStatus.latestStatusTimestamp, max(action_timestamp) as secondLatestStatusTimestamp
                            from fcm_alert_action aa inner join
                                (
                                    select alert_id, create_timestamp, queue_id, max(action_timestamp) as latestStatusTimestamp
                                    from fcm_alert_action aa
                                    where
                                        trunc(TO_DATE(TO_CHAR (aa.action_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from aa.action_timestamp) = l_snap_hr and
                                        alert_status = 'ACTIVE' and
                                        action_type = 'ANALYST ACTION' and
                                        result_id > 0
                                    group by alert_id, create_timestamp, queue_id
                                ) latestStatus
                                on aa.alert_id = latestStatus.alert_id and aa.create_timestamp = latestStatus.create_timestamp and action_timestamp < latestStatus.latestStatusTimestamp
                            group by aa.alert_id, aa.create_timestamp, latestStatus.queue_id, latestStatus.latestStatusTimestamp
                        ) statusedAlerts
                        on faa.alert_id = statusedAlerts.alert_id and faa.create_timestamp = statusedAlerts.create_timestamp and faa.action_timestamp = statusedAlerts.secondLatestStatusTimestamp
                        inner join fcm_queue q on statusedAlerts.queue_id = q.queue_id and q.logical_del_ind = '0'
                        inner join fcm_strategy s on q.strategy_id = s.strategy_id and s.logical_del_ind = '0'
                    group by s.business_unit_id, faa.multi_org_id, s.strategy_id, q.queue_id, faa.alert_type
                ),
                SchedAndUnschedTypes as
                (
                    select s.business_unit_id, latest.multi_org_id, s.strategy_id, q.queue_id, latest.alert_type,
                        sum(
                            case
                                when aa.queue_id = q.queue_id and s.strategy_type_code = 'OUT' and ((trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from aa.next_service_timestamp) > l_snap_hr 
                                							or trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) > l_snap_date)  or alert_status <> 'ACTIVE')
                                    then 1
                                else 0
                            end
                        ) as Scheduled_type_ii_plusUnsch_ii,
                        sum(
                            case
                                when aa.queue_id = q.queue_id and s.strategy_type_code = 'OUT' and ((trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from aa.next_service_timestamp) <= l_snap_hr 
                                							or trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) < l_snap_date)  and alert_status = 'ACTIVE')
                                    then 1
                                else 0
                            end
                        ) as serviceableReviewed,
                        sum(
                            case
                                when aa.queue_id = q.queue_id and s.strategy_type_code in ('EML','LTR','SMS')
                                    then 1
                                else 0
                            end
                        ) as Unscheduled_types_i_and_iii,
                        sum(
                            case
                                when aa.queue_id = q.queue_id and aa.result_id = 6 and s.strategy_type_code = 'OUT' and ((trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from aa.next_service_timestamp) > l_snap_hr or trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) > l_snap_date) or alert_status <> 'ACTIVE')
                                    then 1
                                else 0
                            end
                        ) as verifiedActivity,
                        sum(
                            case
                                when aa.queue_id = q.queue_id and aa.result_id = 7 and s.strategy_type_code = 'OUT' and ((trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from aa.next_service_timestamp) > l_snap_hr or trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) > l_snap_date) or alert_status <> 'ACTIVE')
                                    then 1
                                else 0
                            end
                        ) as confirmedFraud,
                        sum(
                            case
                                when aa.queue_id = q.queue_id and aa.result_id = 7 and s.strategy_type_code = 'OUT' and ((trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from aa.next_service_timestamp) > l_snap_hr or trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) > l_snap_date) or alert_status <> 'ACTIVE')
                                    then aa.open_to_buy_amt
                                else 0
                            end
                        ) as prevented,
                        count(*) as Actioned
                    from
                        (
                            select fcmaa.alert_id, fcmaa.create_timestamp, fcmaa.queue_id, fcmaa.result_id, fcmaa.open_to_buy_amt, fcmaa.next_service_timestamp, fcmaa.alert_origin, fcmaa.alert_status
                            from fcm_alert_action fcmaa
                            inner join
                            (
                                select alert_id, create_timestamp, max(action_timestamp) as maxActionTimestamp
                                from fcm_alert_action
                                where
                                    trunc(TO_DATE(TO_CHAR (action_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) < l_snap_date 
                                    	or trunc(TO_DATE(TO_CHAR (action_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from action_timestamp) <= l_snap_hr
                                group by alert_id, create_timestamp
                            )  latestAlertActionRecords
                            on fcmaa.alert_id = latestAlertActionRecords.alert_id and
                                fcmaa.create_timestamp = latestAlertActionRecords.create_timestamp and
                                fcmaa.action_timestamp = latestAlertActionRecords.maxActionTimestamp
                            inner join fcm_queue q on fcmaa.queue_id = q.queue_id and q.logical_del_ind = '0'
                            inner join fcm_strategy s on q.strategy_id = s.strategy_id and s.logical_del_ind = '0'
                        ) aa inner join
                        (   select a.alert_id, a.create_timestamp, a.queue_id, a.multi_org_id, a.alert_type, max(a.action_timestamp) as latestAction
                            from fcm_alert_action a
                            where (a.alert_id, a.create_timestamp, a.queue_id, a.alert_type) in
                                    (   select alert_id, create_timestamp, queue_id, alert_type
                                        from fcm_alert_action
                                        where
                                            trunc(TO_DATE(TO_CHAR (action_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from action_timestamp) = l_snap_hr and
                                            result_id > 0 and
                                            action_type = 'ANALYST ACTION' and
                                            alert_status = 'ACTIVE') and
                                (trunc(TO_DATE(TO_CHAR (action_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) < l_snap_date or trunc(TO_DATE(TO_CHAR (action_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from action_timestamp) <= l_snap_hr)
                            group by a.alert_id, a.create_timestamp, a.queue_id, a.multi_org_id, a.alert_type
                        ) latest on
                            aa.alert_id = latest.alert_id and
                            aa.create_timestamp = latest.create_timestamp
                        inner join
                            fcm_queue q on latest.queue_id = q.queue_id and q.logical_del_ind = '0' inner join
                            fcm_strategy s on q.strategy_id = s.strategy_id and s.logical_del_ind = '0'
                    group by s.business_unit_id, latest.multi_org_id, s.strategy_id, q.queue_id, latest.alert_type
                ),
                serviceableAlerts as
                (
                    select s.business_unit_id, aa.multi_org_id, s.strategy_id, q.queue_id, aa.alert_type,
                        sum(
                            case
                                when s.strategy_type_code = 'OUT' and (trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from aa.next_service_timestamp) <= l_snap_hr or trunc(TO_DATE(TO_CHAR (aa.next_service_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) < l_snap_date) and alert_status = 'ACTIVE'
                                    then 1
                                else 0
                            end
                        ) as serviceableAlertsCnt,
                        sum(
                            case
                                when trunc(TO_DATE(TO_CHAR (aa.create_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from aa.create_timestamp) = l_snap_hr
                                    then 1
                                else 0
                            end
                        ) as newAlertsCnt
                    from fcm_alert_action aa
                        inner join
                        (
                        select alert_id, create_timestamp, max(action_timestamp) as maxActionTimestamp
                        from fcm_alert_action aa
                        where
                            (trunc(TO_DATE(TO_CHAR (action_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) < l_snap_date 
                            or trunc(TO_DATE(TO_CHAR (action_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from action_timestamp) <= l_snap_hr)
                        group by alert_id, create_timestamp
                        )  latestAlertActionRecords
                        on aa.alert_id = latestAlertActionRecords.alert_id and
                            aa.create_timestamp = latestAlertActionRecords.create_timestamp and
                            aa.action_timestamp = latestAlertActionRecords.maxActionTimestamp
                        inner join fcm_queue q on aa.queue_id = q.queue_id and q.logical_del_ind = '0'
                        inner join fcm_strategy s on q.strategy_id = s.strategy_id and s.logical_del_ind = '0'
                    group by s.business_unit_id, aa.multi_org_id, s.strategy_id, q.queue_id, aa.alert_type
                ),
                unresolvedActions as
                (
                    select
                        s.business_unit_id,
                        a.multi_org_id,
                        s.strategy_id,
                        q.queue_id,
                        a.alert_type,
                        sum(
                            case
                                when a.result_id > 0 and
                                    a.action_type = 'ANALYST ACTION' and
                                    a.alert_status = 'ACTIVE'
                                then 1
                                else 0
                            end
                        ) as connectCnt,
                        sum(
                            case
                                when a.result_id > 0 and
                                    a.action_type = 'ANALYST ACTION' and
                                    a.alert_status = 'ACTIVE'
                                then
                                TO_NUMBER(substr((a.ACTION_TIMESTAMP-a.User_SERVICE_TIMESTAMP),instr((a.ACTION_TIMESTAMP-a.User_SERVICE_TIMESTAMP),' ')+7,2))
                                --sysfun.timestampdiff(2,char(a.ACTION_TIMESTAMP - a.User_SERVICE_TIMESTAMP))
                                else 0
                            end
                        ) as totalTimeSec,
                        sum(
                            case
                                when a.result_id = 2 and
                                    a.action_type = 'ANALYST ACTION' and
                                    a.alert_status = 'ACTIVE'
                                then 1
                                else 0
                            end
                        ) as rescheduleCnt,
                        sum(
                            case
                                when a.result_id = 1 and
                                    a.action_type = 'ANALYST ACTION' and
                                    a.alert_status = 'ACTIVE'
                                then 1
                                else 0
                            end
                        ) as skipCnt,
                        sum(
                            case
                                when a.result_id = 3 and
                                    a.action_type = 'ANALYST ACTION' and
                                    a.alert_status = 'ACTIVE'
                                then 1
                                else 0
                            end
                        ) as pendCnt,
                        sum(
                            case
                                when a.result_id = 4 and
                                    a.action_type = 'ANALYST ACTION' and
                                    a.alert_status = 'ACTIVE'
                                then 1
                                else 0
                            end
                        ) as pendBlockCnt,
                        sum(
                            case
                                when a.result_id = 5 and
                                    a.action_type = 'ANALYST ACTION' and
                                    a.alert_status = 'ACTIVE'
                                then 1
                                else 0
                            end
                        ) as reRouteCnt
                    from
                        fcm_strategy s,
                        fcm_queue q,
                        fcm_alert_action a
                    where
                        q.strategy_id = s.strategy_id and
                        a.queue_id = q.queue_id and
                        trunc(TO_DATE(TO_CHAR (a.action_timestamp, 'YYYY-MON-DD'),'YYYY-MON-DD')) = l_snap_date and extract(hour from a.action_timestamp) = l_snap_hr and
                        s.logical_del_ind = '0' and
                        q.logical_del_ind = '0'
                    group by s.business_unit_id, a.multi_org_id, s.strategy_id, q.queue_id, a.alert_type
                )
                select
                    l_snap_date as snap_date,
                    l_snap_hr as snap_hr,
                    '0' as snap_end_of_day_ind,
                    s.multi_org_id,
                    s.multi_org_name,
                    s.strategy_name,
                    s.queue_name,
                    s.alert_type,
                    s.alert_type_name,
                    coalesce(serviceableAlertsCnt,0) + coalesce(Scheduled_type_ii_plusUnsch_ii, 0) + coalesce(Unscheduled_types_i_and_iii,0) as total_alert_count,
                    case
                        when s.strategy_type_code = 'OUT'
                            then coalesce(serviceableAlertsCnt,0) + coalesce(Scheduled_type_ii_plusUnsch_ii - coalesce(unscheduled_type_ii,0), 0)
                        else 0
                    end as scheduled_alert_count,
                    case
                        when s.strategy_type_code = 'OUT' and coalesce(serviceableAlertsCnt,0) + coalesce(Scheduled_type_ii_plusUnsch_ii - coalesce(unscheduled_type_ii,0), 0) >= coalesce(newAlertsCnt,0)
                            then coalesce(serviceableAlertsCnt,0) + coalesce(Scheduled_type_ii_plusUnsch_ii - coalesce(unscheduled_type_ii,0), 0) - coalesce(newAlertsCnt,0)
                        else 0
                    end as caryover_alert_count,
                    coalesce(newAlertsCnt,0)   as new_alert_count,
                    coalesce(Unscheduled_types_i_and_iii,0) + coalesce(unscheduled_type_ii,0) as unschd_alert_count,
                    coalesce(actioned,0)      as actioned_alert_count,
                    coalesce(Scheduled_type_ii_plusUnsch_ii - coalesce(unscheduled_type_ii,0),0) + coalesce(serviceableReviewed,0) as schd_actioned_alert_count,
                    case
                        when s.strategy_type_code = 'OUT' and coalesce(serviceableAlertsCnt,0) + coalesce(Scheduled_type_ii_plusUnsch_ii - coalesce(unscheduled_type_ii,0), 0) > 0.001
                            then
                                coalesce(cast (cast( ( coalesce(Scheduled_type_ii_plusUnsch_ii - coalesce(unscheduled_type_ii,0),0) + coalesce(serviceableReviewed,0) )  as decimal(19,5) )/ (coalesce(serviceableAlertsCnt,0) + coalesce(Scheduled_type_ii_plusUnsch_ii - coalesce(unscheduled_type_ii,0), 0)) as decimal (19,5) ),0.00)
                        else 0.00
                    end as penetration_rate,
                    coalesce(Unscheduled_types_i_and_iii,0) + coalesce(unscheduled_type_ii,0) as unschd_actioned_alert_count,
                    coalesce(verifiedActivity,0) + coalesce(confirmedFraud,0)    as resolved_alert_count,
                    case
                        when coalesce(actioned,0) > 0.001
                            then
                                cast(coalesce(verifiedActivity,0) + coalesce(confirmedFraud,0) as decimal(19,5)) / actioned
                        else 0
                    end as resolved_alert_rate,
                    coalesce(skipCnt,0)         as skip_count,
                    coalesce(reRouteCnt,0)          as reroute_count,
                    coalesce(rescheduleCnt,0)       as reschd_count,
                    coalesce(pendCnt,0)         as pend_count,
                    coalesce(pendBlockCnt,0)        as pend_block_count,
                    coalesce(verifiedActivity,0)        as verified_ok_count,
                    coalesce(confirmedFraud,0)              as confirmed_fraud_count,
                    case
                        when coalesce(confirmedFraud,0) > 0.001
                            then
                                cast(verifiedActivity as decimal(19,5)) / confirmedFraud
                        else 0
                    end as fp_ratio,
                    coalesce(prevented,0.00)        as saved_amount,
                    coalesce(totalTimeSec,0)       as total_hours_worked,
                    coalesce(serviceableAlertsCnt,0) as serviceable,
                    s.strategy_id,
                    s.queue_id,
                    coalesce(connectCnt,0) as analyst_action_count
                from
                    (select distinct
                        a.multi_org_id,
                        m.multi_org_name,
                        st.strategy_id,
                        st.strategy_name,
                        st.strategy_type_code,
                        q.queue_id,
                        q.queue_name,
                        a.alert_type,
                        at.alert_type_name
                    from
                        fcm_strategy st,
                        fcm_queue q,
                        fcm_alert a,
                        fcm_multi_org m,
                        fcm_alert_type at
                    where
                        st.strategy_id = q.strategy_id
                        and q.queue_id = a.queue_id
                        and a.multi_org_id = m.multi_org_id
                        and a.alert_type = at.alert_type
                        and m.logical_del_ind = '0'
                        and st.logical_del_ind = '0'
                        and q.logical_del_ind = '0'
                        and a.logical_del_ind = '0'
                        and at.logical_del_ind = '0') s
                    left outer join unschTypeii uii
                        on uii.multi_org_id = s.multi_org_id
                        and uii.strategy_id = s.strategy_id and uii.queue_id = s.queue_id and uii.alert_type = s.alert_type
                    left outer join SchedAndUnschedTypes lots
                        on lots.multi_org_id = s.multi_org_id
                        and lots.strategy_id = s.strategy_id and lots.queue_id = s.queue_id and lots.alert_type = s.alert_type
                    left outer join serviceableAlerts svc
                        on svc.multi_org_id = s.multi_org_id
                        and svc.strategy_id = s.strategy_id and svc.queue_id = s.queue_id and svc.alert_type = s.alert_type
                    left outer join unresolvedActions ur
                        on ur.multi_org_id = s.multi_org_id
                        and ur.strategy_id = s.strategy_id and ur.queue_id = s.queue_id and ur.alert_type = s.alert_type
                    ;

            COMMIT;
            ELSE
                v_sqlstate := '99999';
             END IF;

        END LOOP;
--
--

    close cursor1;

END hr_rpt_snapshot;

/
show errors;


